package org.processmining.streamtransitionsystemsminer.algorithms;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import javax.naming.OperationNotSupportedException;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetImpl;
import org.processmining.eventstream.core.interfaces.XSEvent;
import org.processmining.eventstream.readers.abstr.AbstractXSEventReader;
import org.processmining.eventstream.readers.acceptingpetrinet.AcceptingPetriNetXSReaderResultVisualizer;
import org.processmining.eventstream.readers.acceptingpetrinet.XSEventStreamToAcceptingPetriNetReader;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.transitionsystem.regions.TransitionSystem2Petrinet;
import org.processmining.stream.model.datastructure.DSParameterMissingException;
import org.processmining.stream.model.datastructure.DSWrongParameterException;
import org.processmining.stream.model.datastructure.DataStructureFactory;
import org.processmining.stream.model.datastructure.PointerBasedDataStructure;
import org.processmining.stream.reader.views.XSReaderSequenceOfResultsView;
import org.processmining.streamabstractrepresentation.algorithms.UncoupledSizeOneSetTransitionSystemReaderImpl;
import org.processmining.streamabstractrepresentation.parameters.UnCoupledAbstractionSchemeParametersImpl;

public class UncoupledSizeOneSetStreamTransitionSystemsMinerImpl
		extends AbstractXSEventReader<AcceptingPetriNet, AcceptingPetriNet, UnCoupledAbstractionSchemeParametersImpl>
		implements XSEventStreamToAcceptingPetriNetReader {

	private PointerBasedDataStructure<String, Set<XEventClass>> cases;

	private final UncoupledSizeOneSetTransitionSystemReaderImpl<Object, UnCoupledAbstractionSchemeParametersImpl> tsDelegate;
	private final PluginContext context;

	public UncoupledSizeOneSetStreamTransitionSystemsMinerImpl(PluginContext context,
			UnCoupledAbstractionSchemeParametersImpl parameters) {
		super("Stream Transition System Miner (Accepting Petri Net)",
				new XSReaderSequenceOfResultsView<>(
						"Visualizer of: Stream Transition System Miner (Accepting Petri Net)", null,
						parameters.getRefreshRate(), AcceptingPetriNetXSReaderResultVisualizer.instance(context)),
				parameters);
		((XSReaderSequenceOfResultsView<AcceptingPetriNet>) getVisualization()).setReader(this);

		try {
			cases = DataStructureFactory.createPointerDataStructure(parameters.getCaseAdministrationDataStructureType(),
					parameters.getCaseAdministrationDataStructureParameters());
		} catch (OperationNotSupportedException | DSParameterMissingException | DSWrongParameterException e) {
			e.printStackTrace();
			cases = null;
			interrupt();
		}
		tsDelegate = new UncoupledSizeOneSetTransitionSystemReaderImpl<Object, UnCoupledAbstractionSchemeParametersImpl>(
				"ts_stream_delegate", null, parameters, cases, false);
		this.context = context;

	}

	@Override
	public void handleNextPacket(XSEvent event) {
		if (canReadPacket(event)) {
			tsDelegate.handleNextPacket(event);
			super.handleNextPacket(event);
		}

	}

	public void processNewXSEvent(String caseId, XEventClass activity) {
		Set<XEventClass> newSet = new HashSet<>();
		newSet.add(activity);
		cases.add(caseId, newSet);
	}

	protected AcceptingPetriNet computeCurrentResult() {
		TransitionSystem2Petrinet converter = new TransitionSystem2Petrinet();
		try {
			Object[] markedPN = converter.convertToPetrinet(context, tsDelegate.getCurrentResult(), false);
			return new AcceptingPetriNetImpl((Petrinet) markedPN[0], (Marking) markedPN[1]);
		} catch (ConnectionCannotBeObtained | InterruptedException | ExecutionException e) {
			e.printStackTrace();
		}
		return null;

	}

}
